import re
from django import VERSION as django_version
from django.conf import urls
from django.urls import get_resolver
from django.urls import resolvers
from django.utils.deprecation import MiddlewareMixin

from . import utils as maintenance
from .conf import settings

urls.handler503 = "maintenancemode.views.temporary_unavailable"
urls.__all__.append("handler503")

IGNORE_URLS = tuple(re.compile(u) for u in settings.MAINTENANCE_IGNORE_URLS)
DJANGO_VERSION_MAJOR = django_version[0]
DJANGO_VERSION_MINOR = django_version[1]

class MaintenanceModeMiddleware(MiddlewareMixin):
    def process_request(self, request):
        # Allow access if middleware is not activated
        allow_staff = getattr(settings, "MAINTENANCE_ALLOW_STAFF", True)
        allow_superuser = getattr(settings, "MAINTENANCE_ALLOW_SUPERUSER", True)

        if not (settings.MAINTENANCE_MODE or maintenance.status()):
            return None

        INTERNAL_IPS = maintenance.IPList(settings.INTERNAL_IPS)

        # Preferentially check HTTP_X_FORWARDED_FOR b/c a proxy
        # server might have obscured REMOTE_ADDR
        for ip in request.headers.get('X-Forwarded-For', "").split(","):
            if ip.strip() in INTERNAL_IPS:
                return None

        # Allow access if remote ip is in INTERNAL_IPS
        if request.META.get("REMOTE_ADDR") in INTERNAL_IPS:
            return None

        # Allow access if the user doing the request is logged in and a
        # staff member.
        if hasattr(request, "user"):
            if allow_staff and request.user.is_staff:
                return None

            if allow_superuser and request.user.is_superuser:
                return None

        # Check if a path is explicitly excluded from maintenance mode
        for url in IGNORE_URLS:
            if url.match(request.path_info):
                return None
        # Otherwise show the user the 503 page

        if (DJANGO_VERSION_MAJOR == 3 and DJANGO_VERSION_MINOR < 2) or DJANGO_VERSION_MAJOR < 3:
            # Checks if DJANGO version is less than 3.2.0 for breaking change
            resolver = get_resolver()

            callback, param_dict = resolver.resolve_error_handler("503")

            return callback(request, **param_dict)

        else:
            # Default behaviour for django 3.2 and higher
            resolver = resolvers.get_resolver(None)
            resolve = resolver.resolve_error_handler
            callback = resolve('503')

            return callback(request)




from django.core.management.base import BaseCommand, CommandError

from maintenancemode import utils as maintenance


class Command(BaseCommand):
    opts = ("on", "off", "activate", "deactivate")

    def add_arguments(self, parser):
        parser.add_argument("command", nargs="?", help="|".join(self.opts))

    def handle(self, *args, **options):
        command = options.get("command", args[0] if len(args) > 0 else None)
        verbosity = int(options.get("verbosity"))

        if command is not None:
            if command.lower() in ("on", "activate"):
                maintenance.activate()
                if verbosity > 0:
                    self.stdout.write("Maintenance mode was activated succesfully")
            elif command.lower() in ("off", "deactivate"):
                maintenance.deactivate()
                if verbosity > 0:
                    self.stdout.write("Maintenance mode was deactivated succesfully")

        if command not in self.opts:
            raise CommandError("Allowed commands are: %s" % "|".join(self.opts))



"""
    Dummy conftest.py for maintenancemode.
    If you don't know what this is for, just leave it empty.
    Read more about conftest.py under:
    https://pytest.org/latest/plugins.html
"""

import pytest



"""
Django settings for app project.

For more information on this file, see
https://docs.djangoproject.com/en/1.7/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.7/ref/settings/
"""

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
import re

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.7/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "YOUR_SECRET_KEY"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []

# Application definition

PROJECT_APPS = [
    "maintenancemode",
]

INSTALLED_APPS = [
    "django.contrib.auth",
    "django.contrib.admin",
    "django.contrib.sessions",
    "django.contrib.contenttypes",
    "django.contrib.messages",
    "django.contrib.sites",
] + PROJECT_APPS


MIDDLEWARE_CLASSES = [
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    #'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    #'django.middleware.security.SecurityMiddleware',
    "maintenancemode.middleware.MaintenanceModeMiddleware",
]

MIDDLEWARE = MIDDLEWARE_CLASSES

ROOT_URLCONF = "example.urls"

SITE_ID = 1

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [
            os.path.join(os.path.dirname(os.path.abspath(__file__)), "templates"),
        ],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

# Database
# https://docs.djangoproject.com/en/1.7/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": os.path.join(BASE_DIR, "db.sqlite3"),
    }
}

# Internationalization
# https://docs.djangoproject.com/en/1.7/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_L10N = True

USE_TZ = True

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.7/howto/static-files/

# Absolute filesystem path to the directory that will hold user-uploaded files.
MEDIA_ROOT = os.path.join(BASE_DIR, "media")

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash.
MEDIA_URL = "/media/"

STATIC_ROOT = os.path.join(BASE_DIR, "staticfiles")

STATIC_URL = "/static/"


## Maintenance Mode settings

MAINTENANCE_MODE = True  # or ``False`` and use ``maintenance`` command
MAINTENANCE_IGNORE_URLS = (re.compile(r"^/ignored.*"),)



from django.urls import re_path

from django.http import HttpResponse

urlpatterns = [
    re_path("^$", lambda r: HttpResponse("Rendered response page"), name="test"),
    re_path("^ignored/$", lambda r: HttpResponse("Rendered response page"), name="test"),
]



# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
import re

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

SECRET_KEY = "DUMMY_SECRET_KEY"

INTERNAL_IPS = []

# Application definition

PROJECT_APPS = ["maintenancemode.tests", "maintenancemode"]

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.sites",
] + PROJECT_APPS

MIDDLEWARE = [
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "maintenancemode.middleware.MaintenanceModeMiddleware",
]

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [os.path.join(BASE_DIR, "tests", "templates")],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ]
        },
    },
]

ROOT_URLCONF = "maintenancemode.tests.urls"

SITE_ID = 1

# Database
# https://docs.djangoproject.com/en/1.8/ref/settings/#databases

DATABASES = {"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"}}


MAINTENANCE_IGNORE_URLS = (re.compile(r"^/ignored.*"),)



import re

from django.core import management

try:
    from django.utils.six import StringIO
except ImportError:
    from six import StringIO

from django.contrib.auth.models import User
from django.template import TemplateDoesNotExist
from django.test import TestCase
from django.test.client import Client
from django.test.utils import override_settings

from maintenancemode import utils

@override_settings(ROOT_URLCONF="maintenancemode.tests.urls", MAINTENANCE_MODE=False)
class MaintenanceModeMiddlewareTestCase(TestCase):

    def setUp(self):
        utils.deactivate()  # make sure maintenance mode is off

        self.user = User.objects.create_user(
            username="maintenance", email="maintenance@example.org", password="password"
        )

    def tearDown(self):
        self.user.delete()

    def test_default_middleware(self):
        # Middleware should default to being disabled
        response = self.client.get("/")
        self.assertContains(response, text="Rendered response page", count=1, status_code=200)

    def test_disabled_middleware(self):
        # Explicitly disabling the ``MAINTENANCE_MODE`` should work
        with self.settings(MAINTENANCE_MODE=False):
            response = self.client.get("/")
        self.assertContains(response, text="Rendered response page", count=1, status_code=200)

    def test_enabled_middleware_without_template(self):
        # Enabling the middleware without a proper 503 template should
        # raise a template error
        with self.settings(MAINTENANCE_MODE=True, TEMPLATES=[]):
            self.assertRaises(TemplateDoesNotExist, self.client.get, "/")

    def test_enabled_middleware_with_template(self):
        # Enabling the middleware having a ``503.html`` in any of the
        # template locations should return the rendered template"
        with self.settings(MAINTENANCE_MODE=True):
            response = self.client.get("/")
        self.assertContains(response, text="Temporary unavailable", count=1, status_code=503)
        self.assertContains(response, text="You requested: /", count=1, status_code=503)

    def test_middleware_with_non_staff_user(self):
        # A logged in user that is not a staff user should see the 503 message
        self.client.login(username="maintenance", password="password")

        with self.settings(MAINTENANCE_MODE=True):
            response = self.client.get("/")
        self.assertContains(response, text="Temporary unavailable", count=1, status_code=503)

    def test_middleware_with_staff_user(self):
        # A logged in user that _is_ a staff user should be able to
        # use the site normally
        User.objects.filter(pk=self.user.pk).update(is_staff=True)

        self.client.login(username="maintenance", password="password")

        with self.settings(MAINTENANCE_MODE=True):
            response = self.client.get("/")
        self.assertContains(response, text="Rendered response page", count=1, status_code=200)

    def test_middleware_with_staff_user_denied(self):
        # A logged in user that _is_ a staff user should be able to
        # use the site normally
        User.objects.filter(pk=self.user.pk).update(is_staff=True)

        self.client.login(username="maintenance", password="password")

        with self.settings(MAINTENANCE_MODE=True, MAINTENANCE_ALLOW_STAFF=False):
            response = self.client.get("/")
        self.assertContains(response, text="Temporary unavailable", count=1, status_code=503)

    def test_middleware_with_superuser_user_denied(self):
        # A logged in user that _is_ a staff user should be able to
        # use the site normally
        User.objects.filter(pk=self.user.pk).update(is_superuser=True)

        self.client.login(username="maintenance", password="password")

        with self.settings(MAINTENANCE_MODE=True, MAINTENANCE_ALLOW_SUPERUSER=False):
            response = self.client.get("/")
        self.assertContains(response, text="Temporary unavailable", count=1, status_code=503)

    def test_middleware_with_superuser_user_allowed(self):
        # A logged in user that _is_ a staff user should be able to
        # use the site normally
        User.objects.filter(pk=self.user.pk).update(is_superuser=True)

        self.client.login(username="maintenance", password="password")

        with self.settings(MAINTENANCE_MODE=True, MAINTENANCE_ALLOW_STAFF=False):
            response = self.client.get("/")
        self.assertContains(response, text="Rendered response page", count=1, status_code=200)

    def test_middleware_with_internal_ips(self):
        # A user that visits the site from an IP in ``INTERNAL_IPS``
        # should be able to use the site normally

        # Use a new Client instance to be able to set the REMOTE_ADDR used by INTERNAL_IPS
        client = Client(REMOTE_ADDR="127.0.0.1")

        with self.settings(MAINTENANCE_MODE=True, INTERNAL_IPS=("127.0.0.1",)):
            response = client.get("/")
        self.assertContains(response, text="Rendered response page", count=1, status_code=200)

    def test_middleware_with_internal_ips_range(self):
        client = Client(REMOTE_ADDR="10.10.10.1")

        with self.settings(MAINTENANCE_MODE=True, INTERNAL_IPS=("10.10.10.0/24",)):
            response = client.get("/")
        self.assertContains(response, text="Rendered response page", count=1, status_code=200)

    def test_ignored_path(self):
        # A path is ignored when applying the maintanance mode and
        # should be reachable normally
        with self.settings(MAINTENANCE_MODE=True):
            # Note that we cannot override the settings here, since they are
            # ONLY used when the middleware starts up.
            # For this reason, MAINTENANCE_IGNORE_URLS is set in the base
            # settings file.
            response = self.client.get("/ignored/")
        self.assertContains(response, text="Rendered response page", count=1, status_code=200)

    def test_management_command(self):
        out = StringIO()
        # Explicitly disabling the ``MAINTENANCE_MODE``
        with self.settings(MAINTENANCE_MODE=False):
            management.call_command("maintenance", "on", stdout=out)
            self.assertContains(self.client.get("/"), text="Temporary unavailable", count=1, status_code=503)

            management.call_command("maintenance", "off", stdout=out)
            self.assertContains(self.client.get("/"), text="Rendered response page", count=1, status_code=200)



import django

if django.get_version() >= "1.8":
    from django.template.loader import render_to_string
else:
    from django.template import RequestContext, loader

    def render_to_string(template_name, context=None, request=None):
        context_instance = RequestContext(request) if request else None

        return loader.render_to_string(template_name, context, context_instance)


from . import http


def temporary_unavailable(request, template_name="503.html"):
    """
    Default 503 handler, which looks for the requested URL in the
    redirects table, redirects if found, and displays 404 page if not
    redirected.

    Templates: ``503.html``
    Context:
        request_path
            The path of the requested URL (e.g., '/app/pages/bad_page/')

    """
    context = {
        "request_path": request.path,
    }
    return http.HttpResponseTemporaryUnavailable(render_to_string(template_name, context))



import os

from django.conf import settings  # noqa

from appconf import AppConf


class MaintenanceSettings(AppConf):
    IGNORE_URLS = ()
    LOCKFILE_PATH = os.path.join(os.path.abspath(os.path.dirname(__file__)), "maintenance.lock")
    MODE = False

    class Meta:
        prefix = "maintenance"
        holder = "maintenancemode.conf.settings"



import os

from .conf import settings


class IPList(list):
    """Stolen from https://djangosnippets.org/snippets/1362/"""

    def __init__(self, ips):
        try:
            from IPy import IP

            for ip in ips:
                self.append(IP(ip))
        except ImportError:
            pass

    def __contains__(self, ip):
        try:
            for net in self:
                if ip in net:
                    return True
        except:  # noqa
            pass
        return False


def activate():
    try:
        open(settings.MAINTENANCE_LOCKFILE_PATH, "ab", 0).close()
    except OSError:
        pass  # shit happens


def deactivate():
    if os.path.isfile(settings.MAINTENANCE_LOCKFILE_PATH):
        os.remove(settings.MAINTENANCE_LOCKFILE_PATH)


def status():
    return settings.MAINTENANCE_MODE or os.path.isfile(settings.MAINTENANCE_LOCKFILE_PATH)
